SystemOrganization addCategory: #'WidgetTests-Core'!
SystemOrganization addCategory: #'WidgetTests-Support'!
SystemOrganization addCategory: #'WidgetTests-Views'!
SystemOrganization addCategory: #'WidgetTests-Text'!

HandMorph subclass: #UiTestHandMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Support'!

!UiTestHandMorph methodsFor: 'accessing' stamp: 'mt 3/23/2011 13:21:56.721'!
owner: aMorph

	owner := aMorph.! !

!UiTestHandMorph methodsFor: 'private events' stamp: 'mt 3/23/2011 13:30:47.379'!
sendFocusEvent: anEvent to: focusHolder clear: aBlock
	"Send the event to the morph currently holding the focus"

	| result w |	
	w := ActiveWorld.
	w becomeActiveDuring:[
		ActiveHand := self.
		ActiveEvent := anEvent.
		result := focusHolder handleFocusEvent: 
			(anEvent transformedBy: (focusHolder transformedFrom: self))].
		
	^ result! !

TestCase subclass: #UiHtmlTextFormatterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Text'!

!UiHtmlTextFormatterTest methodsFor: 'testing' stamp: 'mt 8/21/2012 11:17:53.57'!
expectedFailures

	^ #(test08overlapping)! !

!UiHtmlTextFormatterTest methodsFor: 'tests' stamp: 'mt 6/22/2010 23:06:07.373'!
test01bold

	| text |
	text := '<b>Hello</b>, World!!'.
	self assert: text equals: text asHtmlText printHtmlString.! !

!UiHtmlTextFormatterTest methodsFor: 'tests' stamp: 'mt 6/22/2010 23:06:48.692'!
test02italic

	| text |
	text := 'Hel<i>lo</i>, W<i>orl</i>d!!'.
	self assert: text equals: text asHtmlText printHtmlString.! !

!UiHtmlTextFormatterTest methodsFor: 'tests' stamp: 'mt 6/22/2010 23:09:02.375'!
test03mixed

	| text |
	text := '<u>Ahahaha!! <b>Hello</b></u>, <i><b>World</b></i>!!'.
	self assert: text ~= text asHtmlText printHtmlString. "Bad Squeak HTML generation."! !

!UiHtmlTextFormatterTest methodsFor: 'tests' stamp: 'mt 6/23/2010 09:04:21.741'!
test04runs

	| runs |
	runs := '<u>A<b>B</b></u>C<i><b>D</b></i>E' asHtmlText runs.
	self
		assert: 5 equals: runs size;
		assert: 1 equals: (runs at: 1) size; "underlined"
		assert: 2 equals: (runs at: 2) size; "underlined, bold"
		assert: 0 equals: (runs at: 3) size;
		assert: 2 equals: (runs at: 4) size; "italic, bold"
		assert: 0 equals: (runs at: 5) size.! !

!UiHtmlTextFormatterTest methodsFor: 'tests' stamp: 'mt 6/23/2010 09:05:50.632'!
test05color

	| text |
	text := '<font color="#FFFF00">A</font>' asHtmlText.
	self assert: Color yellow equals: (text runs first first color)! !

!UiHtmlTextFormatterTest methodsFor: 'tests' stamp: 'mt 10/6/2010 17:08:06.834'!
test06size

	| text |
	text := '<font size="9">A</font>' asHtmlText.
	self assert: 9 equals: (text runs first first font pointSize)! !

!UiHtmlTextFormatterTest methodsFor: 'tests' stamp: 'mt 10/6/2010 17:08:00.418'!
test07sizeAndColor

	| text |
	text := '<font color="#FFFF00" size="9">A</font>' asHtmlText.
	self
		assert: Color yellow equals: (text runs first first color);
		assert: 9 equals: (text runs first second font pointSize)! !

!UiHtmlTextFormatterTest methodsFor: 'tests' stamp: 'mt 7/18/2012 15:36'!
test08overlapping
	"Open two tags at the beginning but close them deferred."

	| text |
	text := '<b><u>Hello</u> World</b>' asHtmlText.
	self assert: text equals: text asHtmlText printHtmlString.! !

TestCase subclass: #UiPropertyItemNodeTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Views'!

!UiPropertyItemNodeTests methodsFor: 'tests' stamp: 'mt 1/25/2013 14:41:24.78'!
test01NewTreeFrom

	| sut |
	sut := UiPropertyItemNode newTreeFrom: {
	#apple -> {
		#green -> {}.
		#yellow -> {}.
		#brown -> {
			#noWorm -> {}.
			#worm -> {}}}.
	#bean -> {
		#lima -> {
			#big -> {}.
			#small -> {}}.
		#unspecified -> {}}.
	#tree -> {
		#needles -> {
			#long -> {}.
			#short -> {}}.
		#leafes -> {}}}.

	self
		assert: 3 equals: sut childCount;
		assert: #apple equals: (sut childAt: 1) data;
		assert: #bean equals: (sut childAt: 2) data;
		assert: #tree equals: (sut childAt: 3) data;
		assert: 3 equals: (sut childAt: 1) childCount;
		assert: 2 equals: (sut childAt: 2) childCount;
		assert: 2 equals: (sut childAt: 3) childCount;
		assert: 2 equals: ((sut childAt: 1) childAt: 3) childCount;
		assert: #worm equals: (((sut childAt: 1) childAt: 3) childAt: 2) data.! !

!UiPropertyItemNodeTests methodsFor: 'tests' stamp: 'mt 1/25/2013 14:39:51.753'!
test02Children

	| sut |
	sut := UiPropertyItemNode new.
	
	self
		assert: 0 equals: sut childCount;
		should: [sut childAt: 1] raise: Error.
		
	sut
		addDefaultGroup;
		addChild: UiPropertyItemNode new.
	
	self
		assert: 1 equals: sut childCount;
		shouldnt: [sut childAt: 1] raise: Error.
! !

!UiPropertyItemNodeTests methodsFor: 'tests' stamp: 'mt 2/9/2012 15:16:02.364'!
test03Value

	| sut |
	sut := UiPropertyItemNode new.
	
	self assert: sut data isNil.
	
	sut data: #foobar.
	
	self assert: #foobar equals: sut data.! !

!UiPropertyItemNodeTests methodsFor: 'tests' stamp: 'mt 1/25/2013 14:40:46.112'!
test04Properties

	| sut |
	sut := UiPropertyItemNode new.
	self assert: sut properties isEmpty.
	
	sut at: #label put: 'Hello, World'.
	self assert: 1 equals: sut properties size.
	
	sut at: #apple put: 'Tasty!!'.
	self
		assert: #label equals: sut properties first;
		assert: #apple equals: sut properties second.
		
	sut removeKey: #label.
	self assert: 1 equals: sut properties size.! !

!UiPropertyItemNodeTests methodsFor: 'tests' stamp: 'mt 1/25/2013 15:09:28.196'!
test05NewTreeFromPaths

	| sut |
	sut := UiPropertyItemNode newTreeFromPaths: #(
		#(apple hole worm)
		#(apple color yellow)
		#(tree leaf bird)
		#(tree leaf green)).
		
	self
		assert: 2 equals: (sut childCount);
		assert: #apple equals: (sut childAt: 1) data;
		assert: #tree equals: (sut childAt: 2) data;
		assert: 2 equals: (sut childAt: 1) childCount;
		assert: 1 equals: (sut childAt: 2) childCount;
		assert: #yellow equals: (((sut childAt: 1) childAt: 2) childAt: 1) data.! !

!UiPropertyItemNodeTests methodsFor: 'tests' stamp: 'mt 1/25/2013 15:08:14.266'!
test06DataProperties

	| sut |
	sut := UiPropertyItemNode new.
	
	self assert: sut properties isEmpty.
	sut data: #foo.
	self assert: #data equals: sut properties first.! !

!UiPropertyItemNodeTests methodsFor: 'tests' stamp: 'mt 1/25/2013 15:28:31.427'!
test07NewTreeFromPathsWithProperties

	| sut |
	sut := UiPropertyItemNode newTreeFromPaths: #(
		#(apple hole worm)
		#(apple color yellow)
		#(tree leaf bird)
		#(tree leaf green)) withProperties: {'default' -> [:node | {#text -> node size. #icon -> MenuIcons blankIcon}]}.
		
	self
		assert: 5 equals: (sut childAt: 1) defaulttext;
		assert: Form equals: (sut childAt: 1) defaulticon class.! !

!UiPropertyItemNodeTests methodsFor: 'tests' stamp: 'mt 1/25/2013 15:32:03.411'!
test08Groups

	| sut |
	sut := UiPropertyItemNode new.
	sut groups: #(first second).
	sut addChild: UiPropertyItemNode new inGroup: #first.

	self
		assert: 2 equals: sut groups size;
		assert: 1 equals: (sut childCountInGroup: #first);
		assert: 0 equals: (sut childCountInGroup: #second).! !

!UiPropertyItemNodeTests methodsFor: 'tests' stamp: 'mt 1/25/2013 15:37:03.586'!
test09PropertyChanged

	| sut spy |
	sut := UiPropertyItemNode new.
	spy := SignalSpy onSender: sut signal: #propertiesChanged:.
	
	sut at: #foo put: #bar.
	self assert: #(#(foo)) equals: spy waitForNextSignal second.! !

!UiPropertyItemNodeTests methodsFor: 'tests' stamp: 'mt 1/25/2013 17:00:57.951'!
test10DeferredAddition

	| sut spy |
	sut := UiPropertyItemNode new.
	spy := SignalSpy onSender: sut signal: #childrenInsertedFrom:to:inGroup:.
	
	sut
		addGroup: #first;
		addGroup: #second;
		deferThreshold: 3;  
		beginDeferAdd;
		addChild: UiPropertyItemNode new inGroup: #first;
		addChild: UiPropertyItemNode new inGroup: #first;
		addChild: UiPropertyItemNode new inGroup: #second.
		
	self
		assert: #(1 2 first) equals: spy waitForNextSignal second;
		assert: #(1 1 second) equals: spy waitForNextSignal second.
		
	sut
		addChild: UiPropertyItemNode new inGroup: #second;
		endDeferAdd.
		
	self assert: #(2 2 second) equals: spy waitForNextSignal second.! !

TestCase subclass: #UiTestCase
	instanceVariableNames: 'sut spy hand time'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Support'!

!UiTestCase commentStamp: 'mt 3/5/2011 08:24:25.856' prior: 0!
Test environment for widgets. Simulates events and may spy on signals triggered by widgets.

Test class name infers the "subject-under-test"!! Empty subclasses will have the same tests as the base class but with different sut object. See #setUp.!

UiTestCase subclass: #UiButtonTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Core'!

!UiButtonTests methodsFor: 'tests' stamp: 'mt 3/5/2011 08:06:51.889'!
test01New

	self
		deny: sut checkable;
		deny: sut checked;
		deny: sut autoExclusive;
		deny: sut down;
		assert: sut enabled.! !

!UiButtonTests methodsFor: 'tests' stamp: 'mt 3/5/2011 08:08:59.399'!
test02Checkable

	sut checked: true.
	
	self deny: sut checked.
	
	sut checkable: true.  
	
	self deny: sut checked.
	
	sut checked: true.
	
	self assert: sut checked.
	
	sut checkable: false.
	
	self deny: sut checked.  ! !

!UiButtonTests methodsFor: 'tests' stamp: 'mt 3/5/2011 08:09:54.45'!
test03Toggle

	sut toggle.
	
	self deny: sut checked.
	
	sut checkable: true.  
	
	self deny: sut checked.
	
	sut toggle.
	
	self assert: sut checked.
	
	sut checkable: false.
	
	self deny: sut checked.  ! !

!UiButtonTests methodsFor: 'tests' stamp: 'mt 3/5/2011 08:17:19.892'!
test04AutoExclusive

	| group otherButton |
	group := Morph new.
	otherButton := sut class new.
	
	group addMorph: sut; addMorph: otherButton.  
	
	sut checkable: true.
	otherButton checkable: true.
	
	sut toggle.
	otherButton toggle.
	
	self
		assert: sut checked;
		assert: otherButton checked.

	sut checkable: false.
	otherButton checkable: false.
		
	sut autoExclusive: true.
	otherButton autoExclusive: true.
	
	self assert: sut checkable.
	
	sut toggle.
	otherButton toggle.
	
	self
		deny: sut checked;
		assert: otherButton checked.
		
	otherButton toggle.
	
	self assert: otherButton checked.! !

!UiButtonTests methodsFor: 'tests - keyboard input' stamp: 'mt 3/5/2011 08:26:25.564'!
testK01ReturnPress

	sut checkable: true.  
	
	self
		deny: sut checked;
		keyPress: Character enter;
		assert: sut checked.  ! !

!UiButtonTests methodsFor: 'tests - mouse input' stamp: 'mt 3/5/2011 08:26:33.684'!
testM01Press

	self deny: sut down.
	
	self
		spyOn: #pressed;
		mousePress: #(left).
	
	self
		assert: sut down;
		assert: #pressed equals: spy waitForNextSignal first.! !

!UiButtonTests methodsFor: 'tests - mouse input' stamp: 'mt 3/5/2011 08:26:38.457'!
testM02Release
	
	sut down: true.
	
	self
		spyOn: #released;
		mouseRelease.
	
	self
		deny: sut down;
		assert: #released equals: spy waitForNextSignal first.! !

!UiButtonTests methodsFor: 'tests - mouse input' stamp: 'mt 3/5/2011 08:26:44.481'!
testM03Click
		
	self
		spyOn: #(pressed released clicked:);
		mouseClick: #(left).
	
	self
		deny: sut down;
		assert: #pressed equals: spy waitForNextSignal first;
		assert: #released equals: spy waitForNextSignal first;
		assert: {#clicked:. {false}} equals: spy waitForNextSignal.! !

!UiButtonTests methodsFor: 'tests - mouse input' stamp: 'mt 3/5/2011 08:26:50.571'!
testM04InteractDisabled
		
	sut
		checkable: true;
		enabled: false.
		
	self
		deny: sut checked;
		mouseClick: #(left);
		deny: sut checked.! !

!UiButtonTests methodsFor: 'tests - mouse input' stamp: 'mt 3/5/2011 08:26:56.618'!
testM05InteractEnabled
		
	sut checkable: true.
		
	self
		deny: sut checked;
		mouseClick: #(left);
		assert: sut checked.! !

!UiButtonTests methodsFor: 'tests - mouse input' stamp: 'mt 3/23/2011 13:35:31.955'!
testM06Toggled
	
	sut checkable: true.
		
	self
		spyOn: #toggled:;
		mouseClick: #(left);
		assert: {#toggled:. {true}} equals: spy waitForNextSignal;
		wait: 400; "avoid double click"
		mouseClick: #(left);
		assert: {#toggled:. {false}} equals: spy waitForNextSignal.! !

UiButtonTests subclass: #UiCheckBoxTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Core'!

UiCheckBoxTests subclass: #UiCheckBoxShiftedTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Core'!

!UiCheckBoxShiftedTests commentStamp: 'mt 3/5/2011 08:54:28.885' prior: 0!
Use another mouse position for events.!

!UiCheckBoxShiftedTests methodsFor: 'events - private' stamp: 'mt 3/5/2011 08:56:22.718'!
simulateMouseEvent: aSymbol at: aPoint buttons: someMouseButtons start: aStartPoint morph: aCheckBox

	super
		simulateMouseEvent: aSymbol
		at: (aCheckBox instVarNamed: #box) center
		buttons: someMouseButtons
		start: aStartPoint
		morph: aCheckBox! !

!UiCheckBoxShiftedTests methodsFor: 'running' stamp: 'mt 3/5/2011 08:54:45.893'!
subjectClass

	^ UiCheckBox! !

!UiCheckBoxTests methodsFor: 'tests' stamp: 'mt 3/5/2011 08:39:38.242'!
test01New

	self
		assert: sut checkable;
		deny: sut checked;
		deny: sut autoExclusive;
		deny: sut down;
		assert: sut enabled.! !

!UiCheckBoxTests methodsFor: 'tests' stamp: 'mt 3/5/2011 08:40:29.345'!
test02Checkable

	sut checked: true.
	
	self assert: sut checked.! !

!UiCheckBoxTests methodsFor: 'tests' stamp: 'mt 3/5/2011 08:41:28.931'!
test03Toggle

	sut toggle.
	
	self assert: sut checked.
	
	sut toggle.
	
	self deny: sut checked.! !

!UiCheckBoxTests methodsFor: 'tests' stamp: 'mt 3/22/2011 08:08:04.907'!
test04SwitchingBehavior

	self
		spyOn: #turnedOn;
		spyOn: #turnedOff.
		
	sut toggle.
	
	self assert: #turnedOn equals: spy waitForNextSignal first.
	
	sut toggle.
	
	self assert: #turnedOff equals: spy waitForNextSignal first.! !

!UiCheckBoxTests methodsFor: 'tests' stamp: 'mt 3/22/2011 08:10:26.25'!
test05SwitchingTimeliness

	| callback state |
	callback := [state := sut checked].
	
	self connect: sut signal: #turnedOn to: callback selector: #value.
	self connect: sut signal: #turnedOff to: callback selector: #value.
		
	sut toggle.
	
	self assert: state.
	
	sut toggle.
	
	self deny: state.! !

!UiCheckBoxTests methodsFor: 'tests - mouse input' stamp: 'mt 3/5/2011 08:45:22.942'!
testM03Click
		
	self
		spyOn: #(pressed released clicked:);
		mouseClick: #(left).
	
	self
		deny: sut down;
		assert: #pressed equals: spy waitForNextSignal first;
		assert: #released equals: spy waitForNextSignal first;
		assert: {#clicked:. {true}} equals: spy waitForNextSignal.! !

UiButtonTests subclass: #UiPushButtonTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Core'!

UiButtonTests subclass: #UiRadioButtonTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Core'!

UiRadioButtonTests subclass: #UiRadioButtonShiftedTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Core'!

!UiRadioButtonShiftedTests commentStamp: 'mt 3/5/2011 12:11:41.409' prior: 0!
Use another mouse position for events.!

!UiRadioButtonShiftedTests methodsFor: 'events - private' stamp: 'mt 3/5/2011 12:11:17.547'!
simulateMouseEvent: aSymbol at: aPoint buttons: someMouseButtons start: aStartPoint morph: aRadioButton

	super
		simulateMouseEvent: aSymbol
		at: (aRadioButton instVarNamed: #box) center
		buttons: someMouseButtons
		start: aStartPoint
		morph: aRadioButton! !

!UiRadioButtonShiftedTests methodsFor: 'running' stamp: 'mt 3/5/2011 12:11:23.45'!
subjectClass

	^ UiRadioButton! !

!UiRadioButtonTests methodsFor: 'tests' stamp: 'mt 3/5/2011 12:06:35.252'!
test01New

	self
		assert: sut checkable;
		deny: sut checked;
		assert: sut autoExclusive;
		deny: sut down;
		assert: sut enabled.! !

!UiRadioButtonTests methodsFor: 'tests' stamp: 'mt 3/5/2011 12:06:44.322'!
test02Checkable

	sut checked: true.
	
	self assert: sut checked.! !

!UiRadioButtonTests methodsFor: 'tests' stamp: 'mt 3/5/2011 12:06:54.738'!
test03Toggle

	sut toggle.
	
	self assert: sut checked.
	
	sut toggle.
	
	self assert: sut checked.! !

!UiRadioButtonTests methodsFor: 'tests - mouse input' stamp: 'mt 3/5/2011 12:09:38.611'!
test04AutoExclusive

	| group otherButton |
	group := Morph new.
	otherButton := sut class new.
	
	group addMorph: sut; addMorph: otherButton.  
	
	sut toggle.
	otherButton toggle.
	
	self
		deny: sut checked;
		assert: otherButton checked.
		
	otherButton toggle.
	
	self
		deny: sut checked;
		assert: otherButton checked.! !

!UiRadioButtonTests methodsFor: 'tests - mouse input' stamp: 'mt 3/5/2011 12:07:12.076'!
testM03Click
		
	self
		spyOn: #(pressed released clicked:);
		mouseClick: #(left).
	
	self
		deny: sut down;
		assert: #pressed equals: spy waitForNextSignal first;
		assert: #released equals: spy waitForNextSignal first;
		assert: {#clicked:. {true}} equals: spy waitForNextSignal.! !

!UiRadioButtonTests methodsFor: 'tests - mouse input' stamp: 'mt 3/5/2011 12:10:18.527'!
testM06Toggled
		
	self
		spyOn: #toggled:;
		mouseClick: #(left).
	
	self assert: {#toggled:. {true}} equals: spy waitForNextSignal.! !

UiButtonTests subclass: #UiToolButtonTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Core'!

!UiToolButtonTests methodsFor: 'tests' stamp: 'mt 2/8/2012 11:32:42.326'!
test01New

	super test01New.
	
	self
		assert: sut autoRaise;
		deny: sut raised;
		assert: #textBesideIcon equals: sut buttonStyle.! !

!UiToolButtonTests methodsFor: 'tests - mouse input' stamp: 'mt 2/8/2012 11:33:04.535'!
testM01AutoRaise

	sut autoRaise: false.
	
	self
		assert: sut raised;
		mouseLeave;
		assert: sut raised.
	
	sut autoRaise: true.
	
	self
		deny: sut raised;
		mouseEnter;
		assert: sut raised;
		mouseLeave;
		deny: sut raised.
		
	sut autoRaise: false.
	
	self assert: sut raised.! !

UiTestCase subclass: #UiEventHandlerTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Core'!

!UiEventHandlerTests commentStamp: 'mt 3/22/2011 18:56:27.578' prior: 0!
Special purpose tests to test event handler.!

!UiEventHandlerTests methodsFor: 'running' stamp: 'mt 3/22/2011 18:53:14.252'!
setUp

	super setUp.
	sut eventHandler: UiEventHandler new.  ! !

!UiEventHandlerTests methodsFor: 'signal spy' stamp: 'mt 3/22/2011 18:55:27.557'!
spyOn: aSignalOrArray
	"Overridden to spy on signals from event handler."
	
	aSignalOrArray isSymbol
		ifFalse: [aSignalOrArray do: [:s | spy addSender: sut eventHandler signal: s]]
		ifTrue: [spy addSender: sut eventHandler signal: aSignalOrArray].! !

!UiEventHandlerTests methodsFor: 'running' stamp: 'mt 3/22/2011 18:56:16.912'!
subjectClass

	^ Morph! !

!UiEventHandlerTests methodsFor: 'tests' stamp: 'mt 3/23/2011 12:26:29.632'!
test01MouseClick

	self
		spyOn: #(mouseClicked: mousePressed: mouseReleased:);
		mouseClick: #(left);
		assert: #mousePressed: equals: spy waitForNextSignal first;
		assert: #mouseClicked: equals: spy waitForNextSignal first;
		assert: #mouseReleased: equals: spy waitForNextSignal first.! !

!UiEventHandlerTests methodsFor: 'tests' stamp: 'mt 3/22/2011 18:57:06.949'!
test02MousePress

	self
		spyOn: #mousePressed:;
		mousePress: #(left);
		assert: #mousePressed: equals: spy waitForNextSignal first.! !

!UiEventHandlerTests methodsFor: 'tests' stamp: 'mt 3/22/2011 22:57:53.467'!
test03MouseRelease

	self flag: #fixme. "does not work because the hand loses mouse focus during event processing...bah"
	self
		spyOn: #mouseReleased:;
		mouseRelease;
		assert: #mouseReleased: equals: spy waitForNextSignal first.! !

!UiEventHandlerTests methodsFor: 'tests' stamp: 'mt 3/22/2011 22:40:16.072'!
test04MouseDoubleClick

	self
		spyOn: #mouseDoubleClicked:;
		mouseDoubleClick: #(left);
		assert: #mouseDoubleClicked: equals: spy waitForNextSignal first.! !

!UiEventHandlerTests methodsFor: 'tests' stamp: 'mt 3/22/2011 22:36:18.625'!
test05KeyStroke

	self
		spyOn: #keyStroked:;
		keyStroke: $c;
		assert: #keyStroked: equals: spy waitForNextSignal first.! !

!UiEventHandlerTests methodsFor: 'tests' stamp: 'mt 3/23/2011 11:50:08.785'!
test06MouseEnter

	self
		spyOn: #mouseEntered:;
		mouseEnter;
		assert: #mouseEntered: equals: spy waitForNextSignal first.! !

!UiEventHandlerTests methodsFor: 'tests' stamp: 'mt 4/7/2011 14:25:49.108'!
test07MouseLeave

	self
		spyOn: #mouseLeft:;
		mouseLeave;
		assert: #mouseLeft: equals: spy waitForNextSignal first.! !

!UiEventHandlerTests methodsFor: 'tests' stamp: 'mt 3/28/2011 10:30:05.639'!
test08SignalConnections

	self
		assert: sut eventHandler signalConnections isEmpty;
		deny: (SignalConnectionsRepository instance includesKey: sut eventHandler);  
		spyOn: #mouseClicked:;
		assert: 1 equals: sut eventHandler signalConnections size;
		deny: (SignalConnectionsRepository instance includesKey: sut eventHandler).
		
	sut eventHandler disconnectAllSignals.
	self assert: sut eventHandler signalConnections isEmpty.
	
	self spyOn: #mouseReleased:.
	sut eventHandler disconnectSignal: #mouseReleased: from: spy.
	self assert: sut eventHandler signalConnections isEmpty.! !

UiTestCase subclass: #UiItemModelFinderTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Views'!

!UiItemModelFinderTests methodsFor: 'running' stamp: 'mt 8/8/2012 11:18:58.923'!
setUp

	super setUp.
	sut root: (UiPropertyItemNode newTreeFrom: {
	#apple -> {
		#green -> {}.
		#yellow -> {}.
		#brown -> {
			#noWorm -> {}.
			#worm -> {}}}.
	#bean -> {
		#lima -> {
			#big -> {}.
			#small -> {}}.
		#unspecified -> {}}.
	#tree -> {
		#needles -> {
			#long -> {}.
			#short -> {}}.
		#leafes -> {}}}).! !

!UiItemModelFinderTests methodsFor: 'tests' stamp: 'mt 2/9/2012 15:21:07.556'!
test01

	"Reserved."! !

!UiItemModelFinderTests methodsFor: 'tests' stamp: 'mt 2/9/2012 15:21:46.585'!
test02DepthFirstDo

	| lengths |
	lengths := OrderedCollection new.

	sut
		traversalMode: #depthFirst;
		do: [:node | lengths add: node data size].
			
	self
		assert: #(5 5 6 5 6 4 4 4 3 5 11 4 7 4 5 6)
		equals: lengths asArray.! !

!UiItemModelFinderTests methodsFor: 'tests' stamp: 'mt 2/9/2012 15:22:20.22'!
test03BreadthFirstDo

	| lengths |
	lengths := OrderedCollection new.

	sut
		traversalMode: #breadthFirst;
		do: [:node | lengths add: node data size].
	
	self
		assert: #(5 4 4 5 6 5 4 11 7 6 6 4 3 5 4 5)
		equals: lengths asArray.! !

!UiItemModelFinderTests methodsFor: 'tests' stamp: 'mt 2/9/2012 15:33:44.866'!
test04Detect

	self
		assert: 0
		equals: (sut detect: [:node | node data = #long]) childCount.! !

!UiItemModelFinderTests methodsFor: 'tests' stamp: 'mt 2/9/2012 15:33:40.008'!
test05DetectNil

	self assert: (sut detect: [:node | node data = #noExisting] ifNone: []) isNil.! !

!UiItemModelFinderTests methodsFor: 'tests' stamp: 'mt 2/9/2012 15:35:49.659'!
test06IteratorNext

	| check |
	check := [:node | node data includesSubString: #orm].
	
	sut traversalMode: #depthFirst.
	
	self
		assert: #noWorm equals: (sut nextSatisfying: check) data;
		assert: #worm equals: (sut nextSatisfying: check) data;
		assert: (sut nextSatisfying: check) isNil;
		assert: sut next isNil.! !

!UiItemModelFinderTests methodsFor: 'tests' stamp: 'mt 2/9/2012 15:37:15.041'!
test07IteratorPrevious

	| check |
	check := [:node | node data includesSubString: #orm].
	
	sut
		traversalMode: #depthFirst;
		nextSatisfying: [:node | node data = #leafes].
	
	self
		assert: sut next isNil;
		assert: #worm equals: (sut previousSatisfying: check) data;
		assert: #noWorm equals: (sut previousSatisfying: check) data;
		assert: (sut previousSatisfying: check) isNil;
		assert: sut previous data isNil."root"! !

!UiItemModelFinderTests methodsFor: 'tests' stamp: 'mt 2/9/2012 15:37:41.562'!
test08New

	self
		assert: sut root equals: sut current;
		assert: #depthFirst equals: sut traversalMode.! !

UiTestCase subclass: #UiListWidgetTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Core'!

!UiListWidgetTests methodsFor: 'tests' stamp: 'mt 4/6/2011 10:03:25.779'!
test01New

	self
		assert: sut isEmpty;
		assert: sut currentIcon isNil;
		assert: sut currentItem isNil;
		assert: sut currentText isNil.! !

!UiListWidgetTests methodsFor: 'tests' stamp: 'mt 4/6/2011 11:22:44.988'!
test02AddItem

	sut addText: 'foobar'.
	
	self
		assert: sut currentItem isNil;
		assert: 1 equals: sut size.
		
	sut addIcon: UiSilkIcons diskIcon text: 'save'.
	
	self assert: 2 equals: sut size.! !

!UiListWidgetTests methodsFor: 'tests' stamp: 'mt 4/6/2011 11:22:19.11'!
test03SelectItem

	sut addText: 'foobar'.
	
	self assert: sut currentItem isNil.
	
	sut currentRow: 1.
	
	self assert: 'foobar' equals: sut currentText asString.! !

!UiListWidgetTests methodsFor: 'tests' stamp: 'mt 4/6/2011 11:23:16.921'!
test04AddItemList

	sut addTextList: #(a b c d e).
	
	self assert: 5 equals: sut size.! !

!UiListWidgetTests methodsFor: 'tests' stamp: 'mt 4/6/2011 11:26:51.449'!
test05RemoveItem

	sut
		addTextList: #(a b c d);
		removeLastRow.
		
	self assert: #c equals: sut items last text asString asSymbol.
	
	sut removeRowAt: 2.
	
	self assert: #(a c) equals: (sut items collect: [:item | item text asString asSymbol]) asArray.! !

!UiListWidgetTests methodsFor: 'tests' stamp: 'mt 6/30/2011 10:47:19.578'!
test06FindItems

	sut addTextList: #(house tree flower dog).
	
	self
		assert: 1 equals: (sut select: [:item | item text asString = 'house']) size;
		assert: 4 equals: (sut select: [:item | item text asString matchesRegex: '.*']) size;
		assert: 3 equals: (sut select: [:item | item text asString includesSubString: 'e']) size.! !

!UiListWidgetTests methodsFor: 'tests' stamp: 'mt 4/6/2011 11:31:36.488'!
test07Clear

	sut
		addTextList: #(a b c d);
		currentRow: 2.
	
	self deny: sut isEmpty.
	
	sut clear.
	
	self
		assert: sut isEmpty;
		assert: sut currentRow isNil.! !

!UiListWidgetTests methodsFor: 'tests' stamp: 'mt 4/6/2011 11:44:06.564'!
test08SelectSortedItems
	"Check for sorting independent model access."
	
	sut
		enableAutoSort;
		addTextList: #(d c b a);
		currentRow: 2.
	
	self assert: #c equals: sut currentText asString asSymbol.! !

UiTestCase subclass: #UiScrollAreaTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Core'!

!UiScrollAreaTests methodsFor: 'tests' stamp: 'mt 3/25/2011 09:00:42.534'!
test01ScrollingWithScrollBars

	sut
		extent: 100@100;
		widget: (Morph new extent: 500@500).
		
	sut fullBounds.
		
	self
		assert: 0 equals: sut horizontalScrollBar currentValue;
		assert: 0 equals: sut verticalScrollBar currentValue;
		assert: 0@0 equals: sut transformMorph offset.
		
	sut horizontalScrollBar currentValue: 100.
	sut verticalScrollBar currentValue: 200.
		
	self assert: 100@200 equals: sut transformMorph offset.! !

!UiScrollAreaTests methodsFor: 'tests' stamp: 'mt 3/25/2011 09:00:53.723'!
test02ScrollingWithOffset

	sut
		extent: 100@100;
		widget: (Morph new extent: 500@500).
		
	sut fullBounds.
		
	self
		assert: 0 equals: sut horizontalScrollBar currentValue;
		assert: 0 equals: sut verticalScrollBar currentValue;
		assert: 0@0 equals: sut transformMorph offset.
		
	sut transformMorph offset: 100@200.
	
	self assert: 100 equals: sut horizontalScrollBar currentValue.
	self assert: 200 equals: sut verticalScrollBar currentValue.! !

!UiScrollAreaTests methodsFor: 'tests' stamp: 'mt 3/28/2011 09:47:37.837'!
test03WidgetResizing

	sut
		horizontalScrollBarPolicy: #alwaysOff;
		verticalScrollBarPolicy: #alwaysOff;
		extent: 100@100;
		widget: (Morph new extent: 100@100).

	sut fullBounds.
	
	self
		assert: 0 equals: sut horizontalScrollBar maximum;
		assert: 0 equals: sut verticalScrollBar maximum.
	
	sut widget extent: 150@150.
	sut transformMorph fullBounds.
	
	self
		assert: 50 equals: sut horizontalScrollBar maximum;
		assert: 50 equals: sut verticalScrollBar maximum.
! !

!UiScrollAreaTests methodsFor: 'tests' stamp: 'mt 3/24/2011 13:47:36.279'!
test04New

	self
		assert: #alwaysOn equals: sut horizontalScrollBarPolicy;  
		assert: #alwaysOn equals: sut verticalScrollBarPolicy;
		assert: sut widget isNil.! !

!UiScrollAreaTests methodsFor: 'tests' stamp: 'mt 3/28/2011 09:47:58.469'!
test05WidgetLayouting

	sut
		horizontalScrollBarPolicy: #alwaysOff;
		verticalScrollBarPolicy: #alwaysOff;
		extent: 100@100;
		widget: (Morph new extent: 100@100).
	
	sut widget
		changeTableLayout;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		addMorph: (Morph new extent: 150@150).
		
	sut transformMorph fullBounds.
		
	self
		assert: 150@150 equals: sut widget extent;
		assert: 50 equals: sut horizontalScrollBar maximum;
		assert: 50 equals: sut verticalScrollBar maximum.
! !

!UiScrollAreaTests methodsFor: 'tests' stamp: 'mt 4/6/2011 13:00:38.252'!
test06WidgetResizable

	sut
		horizontalScrollBarPolicy: #alwaysOff;
		widgetResizable: true;
		extent: 100@100;
		widget: (Morph new extent: 150@150).
	
	sut transformMorph fullBounds.
	
	self assert: sut transformMorph width equals: sut widget width.

	sut
		horizontalScrollBarPolicy: #alwaysOn;  
		verticalScrollBarPolicy: #alwaysOff.
		
	sut transformMorph fullBounds. 
	
	self assert: sut transformMorph height equals: sut widget height.! !

!UiScrollAreaTests methodsFor: 'tests' stamp: 'mt 4/8/2011 10:06:53.619'!
test07AreaResizable

	sut
		extent: 100@100;
		areaResizable: true;
		widget: (Morph new extent: 150@150).

	self
		assert: 100 equals: sut height.
	
	sut
		verticalScrollBarPolicy: #alwaysOff;		
		fullBounds.
	
	self assert: 150 equals: sut transformMorph height.
	
	sut
		verticalScrollBarPolicy: #alwaysOn;
		horizontalScrollBarPolicy: #alwaysOff.
	sut widget width: 200.
	sut fullBounds.
	
	self assert: 200 equals: sut transformMorph width.! !

!UiScrollAreaTests methodsFor: 'tests - mouse input' stamp: 'mt 3/19/2011 10:08:26.383'!
testM01MouseWheel

	sut
		extent: 100@100;
		widget: (Morph new extent: 500@500).
	
	self assert: 0 equals: sut transformMorph verticalOffset.
	
	self
		mouseWheelDown;
		assert: sut verticalScrollBar singleStep equals: sut transformMorph verticalOffset.
		
	self
		mouseWheelUp;
		assert: 0 equals: sut transformMorph verticalOffset.! !

UiTestCase subclass: #UiSliderTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Core'!

!UiSliderTests methodsFor: 'tests' stamp: 'mt 2/25/2011 11:00:25.575'!
test01tracking

	sut
		tracking: true;
		sliderPosition: 0.  
	
	self
		assert: sut hasTracking;
		assert: 0 equals: sut sliderPosition;
		assert: sut sliderPosition = sut currentValue.
	
	sut sliderPosition: 10.
	
	self
		assert: 10 equals: sut sliderPosition;
		assert: sut sliderPosition = sut currentValue.
	
	sut
		tracking: false;
		sliderPosition: 0.
		
	self
		assert: sut hasTracking not;
		assert: sut sliderPosition ~= sut currentValue.    ! !

!UiSliderTests methodsFor: 'tests' stamp: 'mt 2/25/2011 10:56:34.319'!
test02valueChange

	sut
		tracking: true;
		currentValue: 0. 
	
	self
		assert: 0 equals: sut currentValue;
		assert: sut sliderPosition = sut currentValue.
	
	sut currentValue: 10.
	
	self
		assert: 10 equals: sut currentValue;
		assert: sut sliderPosition = sut currentValue.
	
	"Should even work without tracking."
	sut
		tracking: false;
		currentValue: 0.
		
	self assert: sut sliderPosition = sut currentValue.    
	
	sut currentValue: 10.
	
	self assert: sut sliderPosition = sut currentValue.    ! !

!UiSliderTests methodsFor: 'tests' stamp: 'mt 2/25/2011 10:59:37.261'!
test03range

	sut
		minimum: 0;
		maximum: 10.
		
	self assert: 10 equals: sut range extent.
	
	sut minimum: 5.
	
	self assert: 5 equals: sut range extent.
	
	sut range: (1 to: 9).
	
	self
		assert: 1 equals: sut minimum;
		assert: 9 equals: sut maximum;
		assert: 8 equals: sut range extent.! !

!UiSliderTests methodsFor: 'tests' stamp: 'mt 2/25/2011 11:15:09.507'!
test04truncate

	sut
		truncate: false;
		minimum: 0.5;
		maximum: 9.5;
		currentValue: 4.5.
		
	self
		assert: 0.5 equals: sut minimum;
		assert: 9.5 equals: sut maximum;
		assert: 4.5 equals: sut currentValue.
	
	sut truncate: true.

	self
		assert: 1 equals: sut minimum;
		assert: 10 equals: sut maximum;
		assert: 5 equals: sut currentValue.
		
	sut
		minimum: 0.5;
		maximum: 9.5;
		currentValue: 4.5.
		
	self
		assert: 1 equals: sut minimum;
		assert: 10 equals: sut maximum;
		assert: 5 equals: sut currentValue.
! !

UiTestCase subclass: #UiStackedWidgetTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Core'!

!UiStackedWidgetTests methodsFor: 'testing' stamp: 'mt 11/7/2012 16:00:26.277'!
expectedFailures

	^ #(test09CurrentIndexAnimated)! !

!UiStackedWidgetTests methodsFor: 'tests' stamp: 'mt 11/6/2012 11:51:40.223'!
test01AddEmptyPage

	self assert: sut pages isEmpty.
	sut addPage.
	self
		assert: 1 equals: sut pages size;
		assert: sut currentPage == sut pages first.! !

!UiStackedWidgetTests methodsFor: 'tests' stamp: 'mt 11/6/2012 11:52:32.525'!
test02AddPage

	| myPage |
	myPage := Morph new.
	sut addPage: myPage.
	self assert: sut currentPage == myPage.! !

!UiStackedWidgetTests methodsFor: 'tests' stamp: 'mt 11/6/2012 13:22:41.586'!
test03CurrentIndexAndPage

	| page1 page2 |
	page1 := Morph new.
	page2 := Morph new.
	
	sut
		addPage: page1;
		addPage: page2.
		
	self
		assert: 2 equals: sut currentIndex;
		assert: sut currentPage == page2.
	
	sut currentIndex: 1.
	
	self
		assert: 1 equals: sut currentIndex;
		assert: sut currentPage == page1.
! !

!UiStackedWidgetTests methodsFor: 'tests' stamp: 'mt 11/6/2012 11:55:14.506'!
test04EditMode

	self deny: sut editMode.
! !

!UiStackedWidgetTests methodsFor: 'tests' stamp: 'mt 11/6/2012 13:22:58.314'!
test05NextPreviousPage

	| page1 page2 |
	page1 := Morph new.
	page2 := Morph new.
	
	sut
		addPage: page1;
		addPage: page2;
		currentIndex: 1.
		
	self assert: sut currentPage == page1.
	sut nextPage.
	self assert: sut currentPage == page2.
	sut nextPage.
	self assert: sut currentPage == page2.
	sut previousPage.
	self assert: sut currentPage == page1.
	sut previousPage.
	self assert: sut currentPage == page1.! !

!UiStackedWidgetTests methodsFor: 'tests' stamp: 'mt 11/6/2012 13:07:59.482'!
test06RemoveCurrentPage

	| page1 page2 |
	page1 := Morph new.
	page2 := Morph new.
	
	sut
		addPage: page1;
		addPage: page2;
		currentIndex: 2.
		
	sut removeCurrentPage.
	self assert: sut currentPage == page1.
	
	sut removeCurrentPage.
	self assert: sut currentPage isNil.
! !

!UiStackedWidgetTests methodsFor: 'tests' stamp: 'mt 11/7/2012 11:09:00.618'!
test07EditMode

	self
		assert: 0 equals: sut pages size;
		assert: 0 equals: sut submorphs size.

	sut editMode: true.
	
	self
		assert: 1 equals: sut pages size;
		assert: 2 equals: sut submorphs size.
	! !

!UiStackedWidgetTests methodsFor: 'tests' stamp: 'mt 11/7/2012 11:08:35.986'!
test08Clear

	sut addPage.
	self assert: 1 equals: sut pages size.
	sut clear.
	self assert: sut pages isEmpty.
	! !

!UiStackedWidgetTests methodsFor: 'tests' stamp: 'mt 11/7/2012 16:01:11.988'!
test09CurrentIndexAnimated
	"Timing issue. Maybe simulate world cycles for 500 ms. Would make test runs slower though..."

	sut animated: true.
	self test03CurrentIndexAndPage.  ! !

!UiTestCase class methodsFor: 'accessing' stamp: 'mt 2/8/2012 11:25:37.451'!
shouldInheritSelectors

	^ true! !

!UiTestCase methodsFor: 'accessing' stamp: 'mt 3/2/2011 10:26:07.399'!
defaultTimeout

	^ 1 "second"! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/2/2011 11:17:50.817'!
keyPress: aCharacter

	self
		keyPress: sut
		key: aCharacter
		modifiers: #().! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/2/2011 11:15:44.657'!
keyPress: aMorph key: aCharacter modifiers: someButtons

	self
		simulateKeyEvent: #keyDown
		key: aCharacter
		buttons: someButtons
		morph: aMorph.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/2/2011 11:17:41.493'!
keyPress: aCharacter modifiers: someButtons

	self
		keyPress: sut
		key: aCharacter
		modifiers: someButtons.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/2/2011 11:17:23.544'!
keyRelease: aCharacter

	self
		keyRelease: sut
		key: aCharacter
		modifiers: #().! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/2/2011 11:16:56.901'!
keyRelease: aMorph key: aCharacter modifiers: someButtons

	self
		simulateKeyEvent: #keyUp
		key: aCharacter
		buttons: someButtons
		morph: aMorph.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/2/2011 11:17:16.949'!
keyRelease: aCharacter modifiers: someButtons

	self
		keyRelease: sut
		key: aCharacter
		modifiers: someButtons.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/2/2011 11:19:38.535'!
keyStroke: aCharacter

	self
		keyStroke: sut
		key: aCharacter
		modifiers: #().! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/2/2011 11:19:04.99'!
keyStroke: aMorph key: aCharacter modifiers: someButtons

	self
		simulateKeyEvent: #keystroke
		key: aCharacter
		buttons: someButtons
		morph: aMorph.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/2/2011 11:19:26.284'!
keyStroke: aCharacter modifiers: someButtons

	self
		keyStroke: sut
		key: aCharacter
		modifiers: someButtons.! !

!UiTestCase methodsFor: 'events - private' stamp: 'mt 3/2/2011 09:39:21.521'!
mapButton: aSymbol

	^ (Dictionary newFrom: {
		#left   -> 16r04. "red"
		#middle -> 16r01. "blue"
		#right  -> 16r02. "yellow"
		#shift  -> 16r08.
		#ctrl   -> 16r10.
		#cmd    -> 16r40. "alt"
	}) at: aSymbol ifAbsent: [Error signal: 'Unknown button symbol!!']
! !

!UiTestCase methodsFor: 'events - private' stamp: 'mt 3/2/2011 10:19:09.928'!
mapButtons: someMouseButtons
	"Maps symbols to the integer equivalent used by morphic events."
	
	^ someMouseButtons
		inject: 0
		into: [:mask :buttonSymbol | mask bitOr: (self mapButton: buttonSymbol)]! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/2/2011 10:08:47.054'!
mouseClick: someMouseButtons
	"Convenient method for sut."
	
	self mouseClick: sut buttons: someMouseButtons.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/22/2011 22:42:33.278'!
mouseClick: aMorph buttons: someMouseButtons

	self
		mousePress: aMorph buttons: someMouseButtons;
		mouseRelease: aMorph buttons: #().! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/22/2011 22:38:12.065'!
mouseDoubleClick: someMouseButtons
	"Convenient method for sut."
	
	self mouseDoubleClick: sut buttons: someMouseButtons.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/22/2011 22:43:01.017'!
mouseDoubleClick: aMorph buttons: someMouseButtons

	self
		mouseClick: aMorph buttons: someMouseButtons;
		mouseClick: aMorph buttons: someMouseButtons.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/5/2011 08:22:55.718'!
mouseEnter

	self mouseEnter: sut.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/5/2011 08:22:46.645'!
mouseEnter: aMorph

	self
		simulateMouseEvent: #mouseEnter
		at: aMorph center
		buttons: #()
		start: 0@0
		morph: aMorph.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/5/2011 08:23:21.499'!
mouseLeave

	self mouseLeave: sut.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/5/2011 08:23:06.512'!
mouseLeave: aMorph

	self
		simulateMouseEvent: #mouseLeave
		at: aMorph center
		buttons: #()
		start: 0@0
		morph: aMorph.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/2/2011 10:09:01.438'!
mousePress: someMouseButtons

	self mousePress: sut buttons: someMouseButtons! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/2/2011 09:54:12.288'!
mousePress: aMorph buttons: someMouseButtons

	self
		simulateMouseEvent: #mouseDown
		at: aMorph center
		buttons: someMouseButtons
		start: 0@0
		morph: aMorph.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/2/2011 11:18:30.142'!
mouseRelease

	self mouseRelease: #().! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/2/2011 10:09:24.004'!
mouseRelease: someMouseButtons

	self mouseRelease: sut buttons: someMouseButtons.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/2/2011 09:54:32.215'!
mouseRelease: aMorph buttons: someMouseButtons

	self
		simulateMouseEvent: #mouseUp
		at: aMorph center
		buttons: someMouseButtons
		start: 0@0
		morph: aMorph.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/19/2011 09:52:49.019'!
mouseWheel: aMorph direction: aSymbol

	self
		keyStroke: aMorph
		key: (aSymbol = #up
			ifTrue: [Character arrowUp]
			ifFalse: [Character arrowDown])
		modifiers: #(ctrl).! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/19/2011 09:54:03.256'!
mouseWheelDown

	self
		mouseWheel: sut
		direction: #down.! !

!UiTestCase methodsFor: 'events' stamp: 'mt 3/19/2011 09:53:55.101'!
mouseWheelUp

	self
		mouseWheel: sut
		direction: #up.! !

!UiTestCase methodsFor: 'accessing' stamp: 'mt 5/3/2011 13:10:24.472'!
selectedMethod
	"Adaption for CodeSight due to changed implementation of #testSelectors on class-side."
	
	(self class includesSelector: self selector)
		ifTrue: [^ self class >> self selector]
		ifFalse: [self class allSuperclassesDo: [:superclass |
				(superclass includesSelector: self selector)
					ifTrue: [^ superclass >> self selector]]].
			
	Error signal: 'Method could not be found!!'.
	    ! !

!UiTestCase methodsFor: 'running' stamp: 'mt 3/23/2011 13:23:17.839'!
setUp

	super setUp.

	sut := self subjectClass new.
	spy := SignalSpy new.
	hand := UiTestHandMorph new owner: sut.
	time := 0.! !

!UiTestCase methodsFor: 'events - private' stamp: 'mt 3/22/2011 22:20:32.726'!
simulateKeyEvent: aSymbol key: aCharacter buttons: someButtons morph: aMorph

	| morphicEvent |
	morphicEvent := MorphicEvent readFrom: ('{1} {2} {3} {4}' format: {
		aSymbol.
		time.
		self mapButtons: someButtons.
		aCharacter asciiValue}) readStream.
	
	morphicEvent
		wasHandled: false;
		setHand: hand;
		setPosition: aMorph center.
		
	hand newKeyboardFocus: aMorph.
	
	aMorph processEvent: morphicEvent.  
! !

!UiTestCase methodsFor: 'events - private' stamp: 'mt 3/23/2011 13:23:37.853'!
simulateMouseEvent: aSymbol at: aPoint buttons: someMouseButtons start: aStartPoint morph: aMorph

	| morphicEvent |
	morphicEvent := MorphicEvent readFrom: ('{1} {2} {3} {4} {5} {6} {7}' format: {
		aSymbol.
		time.
		aPoint x. aPoint y.
		self mapButtons: someMouseButtons.
		aStartPoint x.
		aStartPoint y}) readStream.
	
	morphicEvent
		wasHandled: false;
		setHand: hand.
	
	hand
		position: aPoint;
		newMouseFocus: aMorph event: morphicEvent;
		handleEvent: morphicEvent.  
! !

!UiTestCase methodsFor: 'signal spy' stamp: 'mt 3/2/2011 10:47:35.438'!
spyOn: aSignalOrArray
	"Convenient method to watch a signal sent by sut."
	
	aSignalOrArray isSymbol
		ifFalse: [aSignalOrArray do: [:s | spy addSender: sut signal: s]]
		ifTrue: [spy addSender: sut signal: aSignalOrArray].! !

!UiTestCase methodsFor: 'running' stamp: 'mt 3/2/2011 10:18:42.758'!
subjectClass
	"Infer subject class from test case class name."
	
	#(Tests TestCase) do: [:suffix |
		(self class name endsWith: suffix)
			ifTrue: [^ Smalltalk at: (self class name allButLast: suffix size) asSymbol]].
		
	self subclassResponsibility.! !

!UiTestCase methodsFor: 'running' stamp: 'mt 6/24/2011 09:38:48.257'!
tearDown

	sut isMorph ifTrue: [sut delete].
	super tearDown.! !

!UiTestCase methodsFor: 'running' stamp: 'mt 3/22/2011 22:01:23.007'!
wait: milliseconds
	"Advance to logic time stamp used by user input events."
	
	time := time + milliseconds.! !

UiTestCase subclass: #UiTextEditTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Core'!

UiTextEditTests subclass: #UiLineEditTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Core'!

!UiLineEditTests methodsFor: 'tests' stamp: 'mt 3/23/2011 11:46:35.988'!
test01New

	self
		deny: sut autoAccept;
		deny: sut autoConvert;
		deny: sut readOnly;
		assert: sut theText = '';
		deny: sut hasUnacceptedChanges;
		assert: sut acceptOnFocusLost;
		assert: sut helpText = ''.! !

!UiLineEditTests methodsFor: 'tests' stamp: 'mt 3/23/2011 10:23:10.171'!
test02HelpText

	sut helpText: 'Type...'.
	
	self assert: 'Type...' equals: sut helpText.! !

!UiLineEditTests methodsFor: 'tests' stamp: 'mt 3/23/2011 10:31:30.289'!
test03SetTextWithLineBreaks

	sut theText: 'Hello
World' asText.

	self assert: 'Hello World' asText equals: sut theText.! !

!UiLineEditTests methodsFor: 'tests' stamp: 'mt 6/24/2011 09:45:23.592'!
test07LayoutingInContainer

	| container anotherSut |
	container := Morph new
		changeTableLayout.
		
	sut
		width: 10;
		hResizing: #spaceFill;
		theText: 'Lorem ipsum blubb top.'.
		
	anotherSut := UiTextEdit new
		width: 10;
		hResizing: #spaceFill;
		theText: 'Lorem ipsum blubb bottom.'.

	container
		addMorphBack: sut;
		addMorphBack: anotherSut.
	
	container fullBounds. "compute layout"

	self assert: sut outerBounds bottom equals: anotherSut outerBounds top.! !

!UiLineEditTests methodsFor: 'tests - keyboard input' stamp: 'mt 3/23/2011 10:40:43.06'!
testK01PressReturn

	self
		spyOn: #(editingFinished: returnPressed:);
		keyStroke: sut textMorph key: $h modifiers: #();
		keyStroke: sut textMorph key: Character cr modifiers: #().
		
	self
		deny: sut hasUnacceptedChanges;
		assert: {#editingFinished:. {'h' asText}} equals: spy waitForNextSignal;
		assert: {#returnPressed:. {'h' asText}} equals: spy waitForNextSignal.! !

!UiLineEditTests methodsFor: 'tests - keyboard input' stamp: 'mt 6/24/2011 09:39:03.52'!
testK02DeferEditSignal

	self
		spyOn: #textEdited:;
		keyStroke: sut textMorph key: $h modifiers: #();
		assert: {#textEdited:. {'h' asText}}
		equals: spy waitForNextSignal.
		
	sut
		bottomRight: 0@0;
		openInWorld; "needs to be in world to have an alarm scheduler; see #alarmScheduler"
		deferEditSignalTime: 50.
	
	self
		assert: spy queue isEmpty;
		keyStroke: sut textMorph key: $h modifiers: #();
		assert: spy queue isEmpty.

	(Delay forMilliseconds: 100) wait.
	World runLocalStepMethods.
	
	self
		assert: {#textEdited:. {'hh' asText}}
		equals: spy waitForNextSignal.! !

!UiTextEditTests methodsFor: 'tests' stamp: 'mt 3/23/2011 09:25:29.734'!
test01New

	self
		deny: sut autoAccept;
		deny: sut autoConvert;
		deny: sut readOnly;
		assert: sut theText = '';
		deny: sut hasUnacceptedChanges;
		deny: sut acceptOnFocusLost.! !

!UiTextEditTests methodsFor: 'tests' stamp: 'mt 3/22/2011 18:26:29.549'!
test02SetText

	self spyOn: #textChanged:.
	
	sut theText: 'Hello, World!!'.
	
	self
		assert: 'Hello, World!!' asText equals: sut theText;
		assert: {#textChanged:. {sut theText}} equals: spy waitForNextSignal.! !

!UiTextEditTests methodsFor: 'tests' stamp: 'mt 3/22/2011 18:28:08.611'!
test03SetTextAsValue

	self
		spyOn: #textChanged:;
		spyOn: #valueChanged:.
	
	sut theText: 42.
	
	self
		assert: '42' asText equals: sut theText;
		assert: {#textChanged:. {sut theText}} equals: spy waitForNextSignal;
		assert: {#valueChanged:. {42}} equals: spy waitForNextSignal.! !

!UiTextEditTests methodsFor: 'tests' stamp: 'mt 5/3/2011 13:00:56.886'!
test04SetTextWithConversion

	self spyOn: #valueChanged:.
	
	sut
		converter: [:raw | raw asString asInteger + 1];
		autoConvert: true;
		theText: 42.
	
	self
		assert: '43' asText equals: sut theText;
		assert: {#valueChanged:. {43}} equals: spy waitForNextSignal.! !

!UiTextEditTests methodsFor: 'tests' stamp: 'mt 3/22/2011 18:37:47.621'!
test05OriginalText

	sut theText: 'Hello, World!!'.
	
	self assert: sut theText equals: sut originalText.! !

!UiTextEditTests methodsFor: 'tests' stamp: 'mt 3/22/2011 18:40:53.63'!
test06SetTextWithReadOnly

	sut
		readOnly: true;
		theText: 'foobar'.
		
	self assert: 'foobar' asText equals: sut theText.! !

!UiTextEditTests methodsFor: 'tests' stamp: 'mt 6/24/2011 09:41:21.989'!
test07LayoutingInContainer

	| container anotherSut |
	container := Morph new
		changeTableLayout.
		
	sut
		width: 10;
		hResizing: #spaceFill;
		theText: 'Lorem ipsum blubb top.';
		areaResizable: true;
		verticalScrollBarPolicy: #alwaysOff.
		
	anotherSut := UiTextEdit new
		width: 10;
		hResizing: #spaceFill;
		theText: 'Lorem ipsum blubb bottom.';
		areaResizable: true;
		verticalScrollBarPolicy: #alwaysOff.

	container
		addMorphBack: sut;
		addMorphBack: anotherSut.
	
	container fullBounds. "compute layout"

	"Layout computation is buggy here. Try to work with it and check here if the bug still exists."
	self deny: sut outerBounds bottom = anotherSut outerBounds top.! !

!UiTextEditTests methodsFor: 'tests - keyboard input' stamp: 'mt 3/23/2011 09:11:59.407'!
testK01EditText

	self
		spyOn: #textEdited:;
		keyStroke: sut textMorph key: $h modifiers: #().
		
	self
		assert: 'h' asText equals: sut theText;
		assert: {#textEdited:. {'h' asText}} equals: spy waitForNextSignal;
		assert: sut hasUnacceptedChanges.
	
	"Save the changes."	
	self
		spyOn: #editingFinished:;
		keyStroke: sut textMorph key: $s modifiers: #(cmd).
		
	self
		assert: {#editingFinished:. {'h' asText}} equals: spy waitForNextSignal;
		deny: sut hasUnacceptedChanges.! !

!UiTextEditTests methodsFor: 'tests - keyboard input' stamp: 'mt 3/23/2011 09:19:23.648'!
testK02EditTextWithAutoAccept

	sut autoAccept: true.

	self
		spyOn: #(textEdited: editingFinished:);
		keyStroke: sut textMorph key: $h modifiers: #().
		
	self
		deny: sut hasUnacceptedChanges;
		assert: {#textEdited:. {'h' asText}} equals: spy waitForNextSignal;
		assert: {#editingFinished:. {'h' asText}} equals: spy waitForNextSignal.! !

!UiTextEditTests methodsFor: 'tests - keyboard input' stamp: 'mt 3/23/2011 10:38:42.384'!
testK03EditTextWithFocusLost

	sut acceptOnFocusLost: false.
	
	self
		spyOn: #editingFinished:;
		keyStroke: sut textMorph key: $h modifiers: #().
		
	sut textMorph
		grabKeyboard;
		releaseKeyboard.
	
	self assert: sut hasUnacceptedChanges.
	
	sut acceptOnFocusLost: true.
	
	sut textMorph
		grabKeyboard;
		releaseKeyboard.
	
	self 
		deny: sut hasUnacceptedChanges;
		assert: {#editingFinished:. {'h' asText}} equals: spy waitForNextSignal.
		
	! !

!UiTextEditTests methodsFor: 'tests - keyboard input' stamp: 'mt 3/23/2011 09:29:28.796'!
testK04ChangeText

	self
		spyOn: #textChanged:;
		keyStroke: sut textMorph key: $h modifiers: #().
		
	self 
		assert: {#textChanged:. {'h' asText}} equals: spy waitForNextSignal.
		
	! !

!UiTextEditTests methodsFor: 'tests - keyboard input' stamp: 'mt 3/23/2011 09:32:43.397'!
testK05ReadOnly

	sut readOnly: true.

	self
		keyStroke: sut textMorph key: $h modifiers: #();
		assert: sut theText = ''.		
	! !

!UiTextEditTests methodsFor: 'tests - keyboard input' stamp: 'mt 5/3/2011 13:01:26.464'!
testK06AutoConvertValue

	sut
		autoConvert: true;
		converter: [:raw | raw asString asInteger + 1].

	self
		spyOn: #valueChanged:;
		keyStroke: sut textMorph key: $5 modifiers: #();
		assert: {#valueChanged:. {6}} equals: spy waitForNextSignal.	
	! !

!UiTextEditTests methodsFor: 'tests - keyboard input' stamp: 'mt 3/23/2011 09:42:22.152'!
testK07AutoConvertText

	sut
		autoConvert: true;
		converter: [:raw | raw asString, 'foobar'].

	self
		spyOn: #textEdited:;
		keyStroke: sut textMorph key: $x modifiers: #();
		assert: {#textEdited:. {'xfoobar' asText}} equals: spy waitForNextSignal;
		assert: spy queue isEmpty.
	! !

!UiTextEditTests methodsFor: 'tests - keyboard input' stamp: 'mt 4/4/2011 14:33:48.819'!
testK08SelectionOnFocusChange

	sut
		grabKeyboard;
		theText: 'Hello, World!!'.
		
	self
		assert: sut hasKeyboard;
		assert: sut textMorph editor selectionInterval isEmpty.
		
	sut textMorph editor selectInterval: (1 to: 5).

	self assert: (1 to: 5) equals: sut textMorph editor selectionInterval.
	
	sut releaseKeyboard.
	TextMorph new grabKeyboard.
	
	self assert: (1 to: 5) equals: sut textMorph editor selectionInterval.! !

!UiTextEditTests methodsFor: 'tests - mouse input' stamp: 'mt 3/22/2011 18:43:54.834'!
testM01GrabKeyboardWithClick

	self
		deny: sut textMorph hasKeyboard;
		mouseClick: #(left);
		assert: sut textMorph hasKeyboard.! !

Object subclass: #UiTestItemModel
	uses: TUiItemModel
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WidgetTests-Support'!

UiTestItemModel class
	uses: TUiItemModel classTrait
	instanceVariableNames: ''!

!UiTestItemModel methodsFor: 'as yet unclassified' stamp: 'mt 6/24/2011 09:55:18.134'!
columnCountFor: aParentModelIndex
	"1st: term; 2nd: length of term"
	
	^ 2! !

!UiTestItemModel methodsFor: 'as yet unclassified' stamp: 'mt 6/24/2011 10:03:28.18'!
indexRow: aRowNumber column: aColumnNumber parent: aParentModelIndex

	| index |
	index := (UiSimpleModelIndex row: aRowNumber column: aColumnNumber)
		model: self;
		parent: aParentModelIndex.
		
	"Header indices."
	(aRowNumber = 0 or: [aColumnNumber = 0])
		ifTrue: [^ index].
		
	aParentModelIndex
		ifNil: [index setProperty: #data toValue: (data at: aRowNumber)]
		ifNotNil: [:i | index setProperty: #data toValue: ((i valueOfProperty: #data) value at: aRowNumber)].
		
	^ index
	! !

!UiTestItemModel methodsFor: 'as yet unclassified' stamp: 'mt 2/8/2012 13:06:33.445'!
initialize

	super initialize.
	
	data := UiPropertyItemNode newFrom: {
	#apple -> {
		#green -> {}.
		#yellow -> {}.
		#brown -> {
			#noWorm -> {}.
			#worm -> {}}}.
	#bean -> {
		#lima -> {
			#big -> {}.
			#small -> {}}.
		#unspecified -> {}}.
	#tree -> {
		#needles -> {
			#long -> {}.
			#short -> {}}.
		#leafes -> {}}}.
! !

!UiTestItemModel methodsFor: 'as yet unclassified' stamp: 'mt 6/24/2011 09:59:14.939'!
rowCountFor: aParentModelIndex

	^ aParentModelIndex
		ifNil: [data size]
		ifNotNil: [(aParentModelIndex valueOfProperty: #data) value size]! !

!UiTestItemModel methodsFor: 'as yet unclassified' stamp: 'mt 6/24/2011 10:04:46.521'!
textFor: aModelIndex

	aModelIndex row = 0
		ifTrue: [^ nil].

	aModelIndex column = 0 ifTrue: [
		^ aModelIndex row asString].
		
	aModelIndex column = 1 ifTrue: [
		^ (aModelIndex valueOfProperty: #data) key].

	aModelIndex column = 2 ifTrue: [
		^ (aModelIndex valueOfProperty: #data) key size].
! !
